function [models, lambda, gamma, obj, utnfac] = trcl_mtmkl(trInd,C,p)
%function [models, lambda, gamma, obj, utnfac] = trcl_mtmkl(trInd,C,p)
%Input:
%GLOBAL VARIABLES
%y - cell of size Tx1. y{t} is column vec. of labels for `t'^{th} task
%K - cell of size Txk. K{t,j} is upper-triangular part of gram-matrix for j^{th} kernel t^{th} task in sparse matrix format
%    (The gram matrices must be PD)
%ARGUMENTS
%trInd - cell of size Tx1. trInd{t} is column vec. of training datapoint
%        indices into gram-matrices in K
%C - common regularization parameter
%p - mixed norm parameter 2 <= p <= inf
%Output:
%models - cell of size Tx1. models{t} is libsvm format model for t^{th} task
%lambda - matrix of lambdas kxT
%gamma - vector of gammas kx1
%obj - optimal objective value
%utnfac - matrix of size Txk containing Unit trace normalization factors

global y
global K 


ITER_MAX = 50;
myEps = 10e-4;
myTol = 10e-8;
T = size(y,1);        %No. tasks
k = size(K,2);        %No. kernels          %if(size(K,1)~=T) fprintf('Invalid y & K\n'); return; end
utnfac = zeros(T,k);

for t=1:T   

    m(t) = size(trInd{t},1);    %No. examples per task
    mpos = sum(y{t}(trInd{t})==1);
    svm_opt{t}=sprintf('-q -t 4 -c %f -w1 %f -w-1 %f', C, (m(t)-mpos)/m(t), mpos/m(t));

    for j=1:k
        utnfac(t,j) = trace(K{1,j}(trInd{t},trInd{t}));
    end
end

if(p<2)
    fprintf('Invalid value of p\n');
    return;
elseif(p==2)
    q = inf;
elseif(p==inf)
    q = 1;
else
    q = p/(p-2);
end
gamma = 1/k*ones(k,1);          %Initialize gamma
if(p~=2)
    lambda = 1/T^(1/q)*ones(k,T);             %Initialize lambda
else
    lambda = ones(k,T);
end
sumQuaTerm = zeros(k,1);
quaTerms = zeros(k,T);

%Loop for gamma
min_obj = 1.7977e+300;
min_models = [];
min_gamma = gamma;
min_lambda = lambda;
obj_gloop = -1.7977e+300;
for iter_gamma = 1:ITER_MAX
    %Solve for alpha, lambda; given gamma
    fprintf('-');
    obj_inner_old = -1.7977e+300;
    for iter_inner = 1:ITER_MAX
        %Update alphas by building T SVMs
        fprintf('.');
        sumLinTerm = 0;
%         'start'
        for t=1:T
            K_comb = (zeros(m(t)));
            for j=1:k
                K_comb = K_comb + (gamma(j)/(lambda(j,t)*utnfac(t,j)))*K{1,j}(trInd{t},trInd{t});
            end
    	    K_comb = K_comb+K_comb'-diag(diag(K_comb));
            models{t} = svmtrain(y{t}(trInd{t}),[(1:m(t))' K_comb],svm_opt{t});

            sumLinTerm = sumLinTerm + sum(abs(models{t}.sv_coef));
            for j=1:k
                quaTerms(j,t) = (models{t}.sv_coef'*K{1,j}(trInd{t}(models{t}.SVs),trInd{t}(models{t}.SVs))*models{t}.sv_coef-0.5*models{t}.sv_coef'*((diag(K{1,j}(trInd{t}(models{t}.SVs),trInd{t}(models{t}.SVs)))).*models{t}.sv_coef))/utnfac(t,j);

            end
        end

        %Update lambdas; given alphas and gamma
        if(p~=2)
            for j=1:k
                lambda(j,:) = quaTerms(j,:).^(1/(q+1))/((sum(quaTerms(j,:).^(q/(q+1))))^(1/q));
            end
        end
        if(T~=1)
            sumQuaTerm=-sum((quaTerms./lambda)')';
        else
            sumQuaTerm=-quaTerms./lambda;
        end
        obj_inner = sumLinTerm + gamma'*sumQuaTerm;
        if(p==2)
            break;
        end
        if(obj_inner_old > obj_inner)
            fprintf('This is bad! obj_gloop=%f, obj_inner=%f, obj_inner_old=%f\n',obj_gloop, obj_inner, obj_inner_old);
%             return;
        elseif((obj_inner-obj_inner_old)/abs(obj_inner_old)<myEps)
            fprintf('convergence inner\n');
            break;
        end
        obj_inner_old = obj_inner;
    end
    if(min_obj > obj_inner)
        min_obj = obj_inner;
        min_models = models;
        min_lambda = lambda;
        min_gamma = gamma;
    end
    if((obj_inner-obj_gloop)/abs(obj_gloop)<myEps)
        fprintf('convergence outer\n');
        break;
    end
    %Gradient computation
    warning off
    pvec = sumQuaTerm.*sqrt(log(k))./(sqrt(iter_gamma)*norm(sumQuaTerm,inf))-1-log(gamma);
    warning on
    gamma = exp(-pvec); gamma = gamma/sum(gamma); gamma(find(gamma<myTol)) = 0;
    %Calculate obj and check for convergence
    obj_gloop = sumLinTerm + gamma'*sumQuaTerm;
end

obj = min_obj;
gamma = min_gamma;
lambda = min_lambda;
models = min_models;
return
